package ircclient

import (
	"net"
	"strconv"
	"strings"
	"testing"
)

func TestConnect(t *testing.T) {
	var config IRCConfig = IRCConfig{Nick: "test",
		Username:       "test",
		Realname:       "testing",
		Password:       "12345",
		ServerPassword: "allow"}
	var listen net.Listener
	var address string

	listen, address = setupSocket()
	var parts []string = strings.Split(address, ":")

	config.Hostname = parts[0]
	config.Port, _ = strconv.Atoi(parts[1])
	go ircServer(listen, t, &config)
	var FromIRC chan IRCMsg

	FromIRC = make(chan IRCMsg)
	config.ReadChannel = FromIRC

	config.Connect()
	defer config.Close()

	var Msg IRCMsg
	var motd, identify bool
	for Msg = range FromIRC {
		if Msg.Cmd == "EndMOTD" {
			t.Log("Got EndMOTD")
			motd = true
		}
		if Msg.Cmd == "Identified" {
			t.Log("Identified")
			identify = true
		}
	}

	if !motd {
		t.Error("Missing EndMOTD")
	}
	if !identify {
		t.Error("Missing Identified")
	}

	if config.MyNick != config.Nick {
		t.Errorf("Got %s, Expected %s", config.MyNick, config.Nick)
	}

}

func TestConnectNickInUse(t *testing.T) {
	var config IRCConfig = IRCConfig{Nick: "bad",
		Username:    "test",
		Realname:    "testing",
		Flood_Num:   1,
		Flood_Delay: 20,
	}
	var listen net.Listener
	var address string

	listen, address = setupSocket()
	var parts []string = strings.Split(address, ":")

	config.Hostname = parts[0]
	config.Port, _ = strconv.Atoi(parts[1])
	go ircServer(listen, t, &config)
	var FromIRC chan IRCMsg

	FromIRC = make(chan IRCMsg)
	config.ReadChannel = FromIRC

	config.Connect()
	defer config.Close()

	var Msg IRCMsg
	var motd, identify bool
	var missing int

	for Msg = range FromIRC {
		if Msg.Cmd == "EndMOTD" {
			t.Log("Got EndMOTD")
			motd = true
			config.WriteTo("missing", "PRIVMSG missing :Missing user")
			config.WriteTo("missing", "PRIVMSG missing :Missing user")
			config.WriteTo("missing", "PRIVMSG missing :Missing user")
			config.WriteTo("#missing", "PRIVMSG #missing :Missing channel")
		}
		if Msg.Cmd == "Identified" {
			t.Log("Identified")
			identify = true
		}
		if Msg.Cmd == "404" || Msg.Cmd == "401" {
			missing++
		}
	}

	if !motd {
		t.Error("Missing EndMOTD")
	}
	if identify {
		t.Error("Should not have been Identified")
	}
	if missing < 2 {
		t.Errorf("Missing should have been 2, was %d", missing)
	}
	if config.MyNick == config.Nick {
		t.Errorf("Nick should be different: Got %s, Didn't Expect %s", config.MyNick, config.Nick)
	}
}
func TestConnectTLS(t *testing.T) {
	var config IRCConfig = IRCConfig{Nick: "test",
		Username:       "test",
		Realname:       "testing",
		Password:       "12345",
		UseTLS:         true,
		UseSASL:        true,
		Insecure:       true,
		ServerPassword: "allow"}
	var listen net.Listener
	var address string

	listen, address = setupTLSSocket()
	var parts []string = strings.Split(address, ":")

	config.Hostname = parts[0]
	config.Port, _ = strconv.Atoi(parts[1])
	go ircServer(listen, t, &config)
	var FromIRC chan IRCMsg

	FromIRC = make(chan IRCMsg)
	config.ReadChannel = FromIRC

	config.Connect()
	defer config.Close()

	var Msg IRCMsg
	var motd, identify bool
	for Msg = range FromIRC {
		if Msg.Cmd == "EndMOTD" {
			t.Log("Got EndMOTD")
			motd = true
		}
		if Msg.Cmd == "Identified" {
			t.Log("Identified")
			identify = true
		}
	}

	if !motd {
		t.Error("Missing EndMOTD")
	}
	if !identify {
		t.Error("Missing Identified")
	}

	if config.MyNick != config.Nick {
		t.Errorf("Got %s, Expected %s", config.MyNick, config.Nick)
	}

}

func TestConnectAutojoin(t *testing.T) {
	var config IRCConfig = IRCConfig{Nick: "test",
		Username:    "test",
		Realname:    "testing",
		Password:    "12345",
		UseTLS:      true,
		UseSASL:     true,
		Insecure:    true,
		AutoJoin:    []string{"#chat", "#test"},
		Flood_Num:   2,
		Flood_Delay: 10,
	}
	var listen net.Listener
	var address string

	listen, address = setupTLSSocket()
	var parts []string = strings.Split(address, ":")

	config.Hostname = parts[0]
	config.Port, _ = strconv.Atoi(parts[1])
	go ircServer(listen, t, &config)
	var FromIRC chan IRCMsg

	FromIRC = make(chan IRCMsg)
	config.ReadChannel = FromIRC

	config.Connect()
	defer config.Close()

	var Msg IRCMsg
	var motd, identify bool
	var joins int
	var expect string
	var ctcpExpect []string = []string{"VERSION",
		"TIME",
		"PING 12345",
	}
	var noticeExpect []string = []string{"Testing",
		"VERSION red-green.com/irc-client",
		"TIME ",
		"PING 12345",
	}

	for Msg = range FromIRC {
		/*
			if (Msg.Cmd == "ACTION") || (Msg.Cmd == "NOTICE") {
				t.Log(Msg)
			}
		*/
		if Msg.Cmd == "EndMOTD" {
			t.Log("Got EndMOTD")
			motd = true
		}
		if Msg.Cmd == "Identified" {
			t.Log("Identified")
			identify = true
		}
		if Msg.Cmd == "JOIN" {
			joins++
			if joins == 2 {
				// messages set to echo are returned to us
				config.WriteTo("echo", "PRIVMSG echo :\x01VERSION\x01")
				config.WriteTo("echo", "PRIVMSG echo :\x01TIME\x01")
				config.WriteTo("echo", "PRIVMSG echo :\x01PING 12345\x01")
				config.Action("echo", "dances.")
				config.Notice("echo", "Testing")
				config.Msg("#test", "Message 1")
				config.Msg("#test", "Message 2")
			}
		}
		if Msg.Cmd == "CTCP" {
			expect = ctcpExpect[0]
			ctcpExpect = ctcpExpect[1:]
			if Msg.Msg != expect {
				t.Errorf("CTCP Got %s, Expected %s", Msg.Msg, expect)
			}
		}
		if Msg.Cmd == "NOTICE" {
			expect = noticeExpect[0]
			if expect != "Testing" {
				expect = "\x01" + expect
			}
			noticeExpect = noticeExpect[1:]
			if !strings.HasPrefix(Msg.Msg, expect) {
				t.Errorf("NOTICE Got [%s], Expected [%s]", Msg.Msg, expect)
			}
		}
		if Msg.Cmd == "ACTION" {
			expect = "dances."
			if Msg.Msg != expect {
				t.Errorf("ACTION Got %s, Expected %s", Msg.Msg, expect)
			}
		}
	}

	if joins != 2 {
		t.Errorf("Expected to autojoin 2 channels, got %d", joins)
	}
	if !motd {
		t.Error("Missing EndMOTD")
	}
	if !identify {
		t.Error("Missing Identified")
	}
	if len(noticeExpect) != 0 {
		t.Errorf("Expected more NOTICEs (%d)", len(noticeExpect))
	}
	if len(ctcpExpect) != 0 {
		t.Errorf("Expected more CTCPs (%d)", len(ctcpExpect))
	}

	if config.MyNick != config.Nick {
		t.Errorf("Got %s, Expected %s", config.MyNick, config.Nick)
	}

}

func TestConnectKick(t *testing.T) {
	var config IRCConfig = IRCConfig{Nick: "test",
		Username:    "test",
		Realname:    "testing",
		Password:    "12345",
		UseTLS:      true,
		UseSASL:     true,
		Insecure:    true,
		AutoJoin:    []string{"#chat", "#test", "#kick"},
		RejoinDelay: 1,
		Flood_Num:   2,
		Flood_Delay: 10,
	}
	var listen net.Listener
	var address string

	listen, address = setupTLSSocket()
	var parts []string = strings.Split(address, ":")

	config.Hostname = parts[0]
	config.Port, _ = strconv.Atoi(parts[1])
	go ircServer(listen, t, &config)
	var FromIRC chan IRCMsg

	FromIRC = make(chan IRCMsg)
	config.ReadChannel = FromIRC

	config.Connect()
	defer config.Close()

	var Msg IRCMsg
	var motd, identify bool
	var joins int

	for Msg = range FromIRC {
		if Msg.Cmd == "EndMOTD" {
			t.Log("Got EndMOTD")
			motd = true
		}
		if Msg.Cmd == "Identified" {
			t.Log("Identified")
			identify = true
		}
		if Msg.Cmd == "JOIN" {
			joins++
		}
	}

	// 3 channels joined, 1 kick, +1 join=4.
	if joins != 4 {
		t.Errorf("Expected to join 4 times, got %d", joins)
	}
	if !motd {
		t.Error("Missing EndMOTD")
	}
	if !identify {
		t.Error("Missing Identified")
	}
}
